@@ -4,10 +4,12 @@ from enum import Enum, EnumMeta
 from os import name
 from pathlib import Path
 from typing import (
+    Iterable,
     List,
     Any,
     Literal,
     Optional,
+    Set,
     Type,
     TypeVar,
     Dict,
@@ -100,7 +102,7 @@ def from_dict(
     clz: Type[T],
     value: Any,
     deserializers: Optional[List[Deserializer]] = None,
-    data: str = "",
+    fpath: str = "",
     ignore_extra_fields: bool = False,
 ) -> T:
     if deserializers is None:
@@ -114,7 +116,7 @@ def from_dict(
 
     ret = False
     for deserializer in deserializers:
-        _value, ok, _ret = deserializer.deserialize(clz, value, data)
+        _value, ok, _ret = deserializer.deserialize(clz, value, fpath)
         if ok:
             value = _value
         ret = ret or _ret
@@ -135,7 +137,7 @@ def from_dict(
         if int(f) == f:
             return int(f)  # type: ignore
         else:
-            raise ValueError(f"Expected {data} to be an int, got {value}")
+            raise ValueError(f"Expected {fpath} to be an int, got {value}")
     elif (
         hasattr(clz, "__args__")
         and len(clz.__args__) == 1  # type: ignore
@@ -158,9 +160,8 @@ def from_dict(
             value = {}
         elif isnamedtupleinstance(value):
             value = value._asdict()
-        assert isinstance(
-            value, dict
-        ), f"{value} cannot be deserialized as dataclass {clz}"
+        if not isinstance(value, dict):
+            raise ValueError(f"'{value}' cannot be deserialized as '{clz.__name__}'.")
         kwargs = {}
         remaining_fields = set(clz.__dataclass_fields__.keys())  # type: ignore
         for field_name, v in value.items():
@@ -177,21 +178,14 @@ def from_dict(
                 clz=field.type,
                 value=v,
                 deserializers=deserializers,
-                data=f"{data}.{field_name}" if data else field_name,
+                fpath=f"{fpath}.{field_name}" if fpath else field_name,
             )
-        for field_name in remaining_fields:
-            field = clz.__dataclass_fields__.get(field_name)  # type: ignore
-            if (
-                field.default is MISSING
-                and field.default_factory is MISSING
-                and is_dataclass(field.type)
-            ):
-                kwargs[field_name] = field.type()
+        _try_create_defaults(clz, kwargs, remaining_fields, fpath)
         try:
             instance = clz(**kwargs)  # type: ignore
             return instance
         except TypeError as e:
-            raise TypeError(f"Failed to initialize {data}: {e}")
+            raise TypeError(f"Failed to initialize '{clz.__name__}': {e}")
     elif isinstance(clz, EnumMeta):
         return clz(value)
     elif typing.get_origin(clz) == Literal:
@@ -199,11 +193,48 @@ def from_dict(
         if value not in args:
             raise ValueError(f"{value} must be one of {args}")
         return value  # type: ignore
+    if isnamedtupleinstance(value) and len(value) == 0:
+        value = value.__class__.__name__
     raise TypeError(
-        f"Failed to deserialize {data}: {value} is not a {_qualified_name(clz)}"
+        f"Failed to deserialize '{fpath}': {repr(value)} is not a {_qualified_name(clz)}"
     )
 
 
+def _try_create_defaults(
+    cls: Any,
+    kwargs: Dict[str, Any],
+    remaining_fields: Iterable[Any],
+    fpath: str,
+) -> Any:
+    still_missing = []
+    for _field_name in remaining_fields:
+        field = cls.__dataclass_fields__.get(_field_name)
+        if field.default is MISSING and field.default_factory is MISSING:
+            if is_dataclass(field.type):
+                _missing_defaults = [
+                    f.name
+                    for f in field.type.__dataclass_fields__.values()
+                    if f.default is MISSING and f.default_factory is MISSING
+                ]
+                _kwargs: Dict[str, Any] = {}
+                _try_create_defaults(
+                    field.type,
+                    _kwargs,
+                    _missing_defaults,
+                    f"{fpath}.{_field_name}" if fpath else _field_name,
+                )
+                kwargs[field.name] = field.type(**_kwargs)
+            else:
+                still_missing.append(_field_name)
+
+    if len(still_missing) > 0:
+        prefix = f"{fpath}." if fpath else ""
+        missing_fields = ", ".join([f"'{prefix}{f}'" for f in still_missing])
+        raise TypeError(
+            f"Failed to initialize '{cls.__name__}': missing value for {missing_fields}"
+        )
+
+
 def loads(
     clz: Type[T],
     data: str,
